package com.example.ellegadodepintia.minijuegoInvestigacion

import android.animation.ObjectAnimator
import android.animation.ValueAnimator
import android.app.Activity
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.ColorMatrix
import android.graphics.ColorMatrixColorFilter
import android.graphics.Matrix
import android.os.Bundle
import android.os.Handler
import android.widget.ImageView
import androidx.appcompat.app.AppCompatActivity
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState
import kotlin.random.Random
import com.example.ellegadodepintia.soundManager.SoundManager

class MinijuegoInvestigar : AppCompatActivity() {
    private lateinit var parte1: ImageView
    private lateinit var parte2: ImageView
    private lateinit var parte3: ImageView
    private lateinit var parte4: ImageView
    private lateinit var soundManager: SoundManager
    private var inicio = false
    private var rotando = false
    private var ultimoClick: Long = 0
    private val angulosRotacion = mutableMapOf<ImageView, Int>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        soundManager = SoundManager(this)
        soundManager.playSoundLoop(R.raw.sound_memory,80)
        setContentView(R.layout.activity_minijuego_investigar)
        inicio = false
        val flag = intent.getBooleanExtra("FLAG_KEY", false) // Si no se encuentra el extra, se usa el valor por defecto (false)
        parte1 = findViewById(R.id.part1)
        parte2 = findViewById(R.id.part2)
        parte3 = findViewById(R.id.part3)
        parte4 = findViewById(R.id.part4)
        angulosRotacion[parte1] = 0
        angulosRotacion[parte2] = 0
        angulosRotacion[parte3] = 0
        angulosRotacion[parte4] = 0
        var originalBitmap: Bitmap? = null
        if(flag){
            originalBitmap = BitmapFactory.decodeResource(resources, R.drawable.asi_exploradores_jabonera)
        }else{
            originalBitmap = BitmapFactory.decodeResource(resources, GameState.eventoActual.imagen)
        }
        val size = minOf(originalBitmap.width, originalBitmap.height)
        val squareBitmap = Bitmap.createBitmap(originalBitmap, 0, 0, size, size)
        val halfSize = size / 2
        val topLeft = Bitmap.createBitmap(squareBitmap, 0, 0, halfSize, halfSize)
        val topRight = Bitmap.createBitmap(squareBitmap, halfSize, 0, halfSize, halfSize)
        val bottomLeft = Bitmap.createBitmap(squareBitmap, 0, halfSize, halfSize, halfSize)
        val bottomRight = Bitmap.createBitmap(squareBitmap, halfSize, halfSize, halfSize, halfSize)

        parte1.setImageBitmap(topLeft)
        parte2.setImageBitmap(topRight)
        parte3.setImageBitmap(bottomLeft)
        parte4.setImageBitmap(bottomRight)

        parte1.setOnClickListener {
            val currentTime = System.currentTimeMillis()
            if (currentTime - ultimoClick >= 300 && !rotando) {
                ultimoClick = currentTime
                rotarImagen(parte1, topLeft)
            }
        }

        parte2.setOnClickListener {
            val currentTime = System.currentTimeMillis()
            if (currentTime - ultimoClick >= 300 && !rotando) {
                ultimoClick = currentTime
                rotarImagen(parte2, topRight)
            }
        }

        parte3.setOnClickListener {
            val currentTime = System.currentTimeMillis()
            if (currentTime - ultimoClick >= 300 && !rotando) {
                ultimoClick = currentTime
                rotarImagen(parte3, bottomLeft)
            }
        }

        parte4.setOnClickListener {
            val currentTime = System.currentTimeMillis()
            if (currentTime - ultimoClick >= 300 && !rotando) {
                ultimoClick = currentTime
                rotarImagen(parte4, bottomRight)
            }
        }
        val randomRotations1 = Random.nextInt(1, 4)
        val randomRotations2 = Random.nextInt(1, 3)
        val randomRotations3 = Random.nextInt(1, 3)
        val randomRotations4 = Random.nextInt(1, 4)

        for (i in 1..randomRotations1) {
            rotarImagen(parte1, topLeft)
        }

        for (i in 1..randomRotations2) {
            rotarImagen(parte2, topRight)
        }

        for (i in 1..randomRotations3) {
            rotarImagen(parte3, bottomLeft)
        }

        for (i in 1..randomRotations4) {
            rotarImagen(parte4, bottomRight)
        }

        inicio = true
        efectoFadeIn(parte1)
        efectoFadeIn(parte2)
        efectoFadeIn(parte3)
        efectoFadeIn(parte4)
    }

    private fun efectoFadeIn(imageView: ImageView) {
        val fadeInAnimator = ObjectAnimator.ofFloat(imageView, "alpha", 0f, 1f)
        fadeInAnimator.duration = 2000
        fadeInAnimator.start()
    }

    private fun rotarImagen(imageView: ImageView, originalBitmap: Bitmap) {
        if(!rotando || !inicio) {
            rotando = true
            soundManager.playSound(R.raw.sound_button,100)
            val anguloActual = angulosRotacion[imageView] ?: 0
            val nuevoAngulo = (anguloActual + 90)
            val rotateAnimator = ObjectAnimator.ofFloat(
                imageView,
                "rotation",
                anguloActual.toFloat(),
                nuevoAngulo.toFloat()
            )
            rotateAnimator.duration = 300
            rotateAnimator.start()
            val matrix = Matrix()
            matrix.postRotate(90f)
            val rotatedBitmap = Bitmap.createBitmap(
                originalBitmap,
                0,
                0,
                originalBitmap.width,
                originalBitmap.height,
                matrix,
                true
            )
            imageView.setImageBitmap(rotatedBitmap)
            angulosRotacion[imageView] = nuevoAngulo
            Handler().postDelayed({
                rotando = false
                comprobarImagenCompleta()
            }, 400)
        }
    }

    private fun comprobarImagenCompleta() {
        val allCorrect = angulosRotacion.values.all { it % 360 == 270}
        println("Ángulos actuales: ${angulosRotacion.values.joinToString(", ")}")
        if (allCorrect && inicio) {
            bloquearPuzle()
        }
    }

    private fun bloquearPuzle() {
        parte1.isEnabled = false
        parte2.isEnabled = false
        parte3.isEnabled = false
        parte4.isEnabled = false
        Handler().postDelayed({
            iluminarParte(parte1)
            iluminarParte(parte2)
            iluminarParte(parte3)
            iluminarParte(parte4)
        }, 500)
        val resultIntent = Intent()
        resultIntent.putExtra("resultado", "Investigación completada")
        setResult(Activity.RESULT_OK, resultIntent)
        soundManager.stopSound()
        finish()
    }

    private fun iluminarParte(imageView: ImageView) {
        val matrix = ColorMatrix()
        matrix.setScale(1.5f, 1.5f, 1.5f, 1f)
        val colorFilter = ColorMatrixColorFilter(matrix)
        imageView.colorFilter = colorFilter
        val animator = ValueAnimator.ofFloat(1.5f, 1f)
        animator.duration = 2000
        animator.addUpdateListener { animation ->
            val value = animation.animatedValue as Float
            val scaleMatrix = ColorMatrix()
            scaleMatrix.setScale(value, value, value, 1f)
            imageView.colorFilter = ColorMatrixColorFilter(scaleMatrix)
        }
        animator.start()
    }
    override fun onPause() {
        super.onPause()
        soundManager.stopSound()
    }
    override fun onBackPressed() {
        val flag = intent.getBooleanExtra("FLAG_KEY", false)
        if (flag){
            super.onBackPressed()
        }
    }
}
